@@ -21,8 +21,8 @@
 
 import enum
 import itertools
-import typing
 import functools
+import typing
 
 import attr
 from PyQt5.QtCore import (pyqtSignal, pyqtSlot, QUrl, QObject, QSizeF, Qt,
@@ -71,7 +71,7 @@ def create(win_id: int,
     mode_manager = modeman.instance(win_id)
     if objects.backend == usertypes.Backend.QtWebEngine:
         from qutebrowser.browser.webengine import webenginetab
-        tab_class = webenginetab.WebEngineTab
+        tab_class = webenginetab.WebEngineTab  # type: typing.Type[AbstractTab]
     else:
         from qutebrowser.browser.webkit import webkittab
         tab_class = webkittab.WebKitTab
@@ -317,6 +317,7 @@ class AbstractSearch(QObject):
     def search(self, text: str, *,
                ignore_case: usertypes.IgnoreCase = usertypes.IgnoreCase.never,
                reverse: bool = False,
+               wrap: bool = True,
                result_cb: _Callback = None) -> None:
         """Find the given text on the page.
 
@@ -324,6 +325,7 @@ class AbstractSearch(QObject):
             text: The text to search for.
             ignore_case: Search case-insensitively.
             reverse: Reverse search direction.
+            wrap: Allow wrapping at the top or bottom of the page.
             result_cb: Called with a bool indicating whether a match was found.
         """
         raise NotImplementedError
@@ -425,27 +427,36 @@ class AbstractZoom(QObject):
         self._set_factor_internal(self._zoom_factor)
 
 
+class SelectionState(enum.Enum):
+
+    """Possible states of selection in caret mode.
+
+    NOTE: Names need to line up with SelectionState in caret.js!
+    """
+
+    none = 1
+    normal = 2
+    line = 3
+
+
 class AbstractCaret(QObject):
 
     """Attribute ``caret`` of AbstractTab for caret browsing."""
 
     #: Signal emitted when the selection was toggled.
-    #: (argument - whether the selection is now active)
-    selection_toggled = pyqtSignal(bool)
+    selection_toggled = pyqtSignal(SelectionState)
     #: Emitted when a ``follow_selection`` action is done.
     follow_selected_done = pyqtSignal()
 
     def __init__(self,
-                 tab: 'AbstractTab',
                  mode_manager: modeman.ModeManager,
                  parent: QWidget = None) -> None:
         super().__init__(parent)
-        self._tab = tab
         self._widget = typing.cast(QWidget, None)
-        self.selection_enabled = False
         self._mode_manager = mode_manager
         mode_manager.entered.connect(self._on_mode_entered)
         mode_manager.left.connect(self._on_mode_left)
+        # self._tab is set by subclasses so mypy knows its concrete type.
 
     def _on_mode_entered(self, mode: usertypes.KeyMode) -> None:
         raise NotImplementedError
@@ -498,7 +509,7 @@ class AbstractCaret(QObject):
     def move_to_end_of_document(self) -> None:
         raise NotImplementedError
 
-    def toggle_selection(self) -> None:
+    def toggle_selection(self, line: bool = False) -> None:
         raise NotImplementedError
 
     def drop_selection(self) -> None:
@@ -540,7 +551,7 @@ class AbstractScroller(QObject):
 
     @pyqtSlot()
     def _log_scroll_pos_change(self) -> None:
-        log.webview.vdebug(  # type: ignore
+        log.webview.vdebug(  # type: ignore[attr-defined]
             "Scroll position changed to {}".format(self.pos_px()))
 
     def _init_widget(self, widget: QWidget) -> None:
@@ -687,9 +698,9 @@ class AbstractElements:
         [typing.Optional['webelem.AbstractWebElement']], None]
     _ErrorCallback = typing.Callable[[Exception], None]
 
-    def __init__(self, tab: 'AbstractTab') -> None:
+    def __init__(self) -> None:
         self._widget = typing.cast(QWidget, None)
-        self._tab = tab
+        # self._tab is set by subclasses so mypy knows its concrete type.
 
     def find_css(self, selector: str,
                  callback: _MultiCallback,
@@ -824,6 +835,15 @@ class AbstractTabPrivate:
     def shutdown(self) -> None:
         raise NotImplementedError
 
+    def run_js_sync(self, code: str) -> None:
+        """Run javascript sync.
+
+        Result will be returned when running JS is complete.
+        This is only implemented for QtWebKit.
+        For QtWebEngine, always raises UnsupportedOperationError.
+        """
+        raise NotImplementedError
+
 
 class AbstractTab(QWidget):
 
@@ -867,8 +887,18 @@ class AbstractTab(QWidget):
     # arg 1: The exit code.
     renderer_process_terminated = pyqtSignal(TerminationStatus, int)
 
-    def __init__(self, *, win_id: int, private: bool,
+    # Hosts for which a certificate error happened. Shared between all tabs.
+    #
+    # Note that we remember hosts here, without scheme/port:
+    # QtWebEngine/Chromium also only remembers hostnames, and certificates are
+    # for a given hostname anyways.
+    _insecure_hosts = set()  # type: typing.Set[str]
+
+    def __init__(self, *, win_id: int,
+                 mode_manager: modeman.ModeManager,
+                 private: bool,
                  parent: QWidget = None) -> None:
+        utils.unused(mode_manager)  # needed for mypy
         self.is_private = private
         self.win_id = win_id
         self.tab_id = next(tab_id_gen)
@@ -884,7 +914,6 @@ class AbstractTab(QWidget):
         self._layout = miscwidgets.WrapperLayout(self)
         self._widget = typing.cast(QWidget, None)
         self._progress = 0
-        self._has_ssl_errors = False
         self._load_status = usertypes.LoadStatus.none
         self._tab_event_filter = eventfilter.TabEventFilter(
             self, parent=self)
@@ -946,7 +975,7 @@ class AbstractTab(QWidget):
             log.webview.warning("Unable to find event target!")
             return
 
-        evt.posted = True
+        evt.posted = True  # type: ignore[attr-defined]
         QApplication.postEvent(recipient, evt)
 
     def navigation_blocked(self) -> bool:
@@ -971,7 +1000,6 @@ class AbstractTab(QWidget):
     @pyqtSlot()
     def _on_load_started(self) -> None:
         self._progress = 0
-        self._has_ssl_errors = False
         self.data.viewing_source = False
         self._set_load_status(usertypes.LoadStatus.loading)
         self.load_started.emit()
@@ -1030,15 +1058,19 @@ class AbstractTab(QWidget):
         Needs to be called by subclasses to trigger a load status update, e.g.
         as a response to a loadFinished signal.
         """
-        if ok and not self._has_ssl_errors:
-            if self.url().scheme() == 'https':
-                self._set_load_status(usertypes.LoadStatus.success_https)
-            else:
-                self._set_load_status(usertypes.LoadStatus.success)
-        elif ok:
-            self._set_load_status(usertypes.LoadStatus.warn)
+        url = self.url()
+        is_https = url.scheme() == 'https'
+
+        if not ok:
+            loadstatus = usertypes.LoadStatus.error
+        elif is_https and url.host() in self._insecure_hosts:
+            loadstatus = usertypes.LoadStatus.warn
+        elif is_https:
+            loadstatus = usertypes.LoadStatus.success_https
         else:
-            self._set_load_status(usertypes.LoadStatus.error)
+            loadstatus = usertypes.LoadStatus.success
+
+        self._set_load_status(loadstatus)
 
     @pyqtSlot()
     def _on_history_trigger(self) -> None:
@@ -1126,7 +1158,8 @@ class AbstractTab(QWidget):
     def __repr__(self) -> str:
         try:
             qurl = self.url()
-            url = qurl.toDisplayString(QUrl.EncodeUnicode)  # type: ignore
+            url = qurl.toDisplayString(
+                QUrl.EncodeUnicode)  # type: ignore[arg-type]
         except (AttributeError, RuntimeError) as exc:
             url = '<{}>'.format(exc.__class__.__name__)
         else:
