@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -22,14 +22,16 @@
 import os
 import os.path
 import tempfile
+import typing
 
 from PyQt5.QtCore import pyqtSignal, pyqtSlot, QObject, QSocketNotifier
 
 from qutebrowser.utils import message, log, objreg, standarddir, utils
 from qutebrowser.commands import runners
-from qutebrowser.config import config
+from qutebrowser.config import websettings
 from qutebrowser.misc import guiprocess
 from qutebrowser.browser import downloads
+from qutebrowser.qt import sip
 
 
 class _QtFIFOReader(QObject):
@@ -58,8 +60,10 @@ class _QtFIFOReader(QObject):
         fd = os.open(filepath, os.O_RDWR | os.O_NONBLOCK)
         # pylint: enable=no-member,useless-suppression
         self._fifo = os.fdopen(fd, 'r')
-        self._notifier = QSocketNotifier(fd, QSocketNotifier.Read, self)
-        self._notifier.activated.connect(self.read_line)
+        self._notifier = QSocketNotifier(typing.cast(sip.voidptr, fd),
+                                         QSocketNotifier.Read, self)
+        self._notifier.activated.connect(  # type: ignore[attr-defined]
+            self.read_line)
 
     @pyqtSlot()
     def read_line(self):
@@ -113,11 +117,11 @@ class _BaseUserscriptRunner(QObject):
         self._cleaned_up = False
         self._filepath = None
         self._proc = None
-        self._env = {}
+        self._env = {}  # type: typing.MutableMapping[str, str]
         self._text_stored = False
         self._html_stored = False
-        self._args = None
-        self._kwargs = None
+        self._args = ()  # type: typing.Tuple[typing.Any, ...]
+        self._kwargs = {}
 
     def store_text(self, text):
         """Called as callback when the text is ready from the web backend."""
@@ -147,7 +151,8 @@ class _BaseUserscriptRunner(QObject):
             log.procs.debug("Both text/HTML stored, kicking off userscript!")
             self._run_process(*self._args, **self._kwargs)
 
-    def _run_process(self, cmd, *args, env=None, verbose=False):
+    def _run_process(self, cmd, *args, env=None, verbose=False,
+                     output_messages=False):
         """Start the given command.
 
         Args:
@@ -155,13 +160,16 @@ class _BaseUserscriptRunner(QObject):
             *args: The arguments to hand to the command
             env: A dictionary of environment variables to add.
             verbose: Show notifications when the command started/exited.
+            output_messages: Show the output as messages.
         """
+        assert self._filepath is not None
         self._env['QUTE_FIFO'] = self._filepath
         if env is not None:
             self._env.update(env)
-        self._proc = guiprocess.GUIProcess('userscript',
-                                           additional_env=self._env,
-                                           verbose=verbose, parent=self)
+
+        self._proc = guiprocess.GUIProcess(
+            'userscript', additional_env=self._env,
+            output_messages=output_messages, verbose=verbose, parent=self)
         self._proc.finished.connect(self.on_proc_finished)
         self._proc.error.connect(self.on_proc_error)
         self._proc.start(cmd, args)
@@ -170,12 +178,15 @@ class _BaseUserscriptRunner(QObject):
         """Clean up temporary files."""
         if self._cleaned_up:
             return
+        assert self._filepath is not None
         self._cleaned_up = True
+
         tempfiles = [self._filepath]
         if 'QUTE_HTML' in self._env:
             tempfiles.append(self._env['QUTE_HTML'])
         if 'QUTE_TEXT' in self._env:
             tempfiles.append(self._env['QUTE_TEXT'])
+
         for fn in tempfiles:
             log.procs.debug("Deleting temporary file {}.".format(fn))
             try:
@@ -185,6 +196,7 @@ class _BaseUserscriptRunner(QObject):
                 # executed async.
                 message.error("Failed to delete tempfile {} ({})!".format(
                     fn, e))
+
         self._filepath = None
         self._proc = None
         self._env = {}
@@ -254,7 +266,7 @@ class _POSIXUserscriptRunner(_BaseUserscriptRunner):
             return
 
         self._reader = _QtFIFOReader(self._filepath)
-        self._reader.got_line.connect(self.got_cmd)
+        self._reader.got_line.connect(self.got_cmd)  # type: ignore[arg-type]
 
     @pyqtSlot()
     def on_proc_finished(self):
@@ -268,6 +280,8 @@ class _POSIXUserscriptRunner(_BaseUserscriptRunner):
         """Clean up reader and temporary files."""
         if self._cleaned_up:
             return
+        assert self._reader is not None
+
         log.procs.debug("Cleaning up")
         self._reader.cleanup()
         self._reader.deleteLater()
@@ -293,6 +307,7 @@ class _WindowsUserscriptRunner(_BaseUserscriptRunner):
         """Clean up temporary files after the userscript finished."""
         if self._cleaned_up:
             return
+        assert self._filepath is not None
 
         try:
             with open(self._filepath, 'r', encoding='utf-8') as f:
@@ -388,7 +403,8 @@ def _lookup_path(cmd):
     raise NotFoundError(cmd, directories)
 
 
-def run_async(tab, cmd, *args, win_id, env, verbose=False):
+def run_async(tab, cmd, *args, win_id, env, verbose=False,
+              output_messages=False):
     """Run a userscript after dumping page html/source.
 
     Raises:
@@ -403,15 +419,15 @@ def run_async(tab, cmd, *args, win_id, env, verbose=False):
         win_id: The window id the userscript is executed in.
         env: A dictionary of variables to add to the process environment.
         verbose: Show notifications when the command started/exited.
+        output_messages: Show the output as messages.
     """
-    tabbed_browser = objreg.get('tabbed-browser', scope='window',
-                                window=win_id)
-    commandrunner = runners.CommandRunner(win_id, parent=tabbed_browser)
+    tb = objreg.get('tabbed-browser', scope='window', window=win_id)
+    commandrunner = runners.CommandRunner(win_id, parent=tb)
 
     if utils.is_posix:
-        runner = _POSIXUserscriptRunner(tabbed_browser)
+        runner = _POSIXUserscriptRunner(tb)  # type: _BaseUserscriptRunner
     elif utils.is_windows:  # pragma: no cover
-        runner = _WindowsUserscriptRunner(tabbed_browser)
+        runner = _WindowsUserscriptRunner(tb)
     else:  # pragma: no cover
         raise UnsupportedError
 
@@ -419,10 +435,8 @@ def run_async(tab, cmd, *args, win_id, env, verbose=False):
         lambda cmd:
         log.commands.debug("Got userscript command: {}".format(cmd)))
     runner.got_cmd.connect(commandrunner.run_safely)
-    user_agent = config.val.content.headers.user_agent
-    if user_agent is not None:
-        env['QUTE_USER_AGENT'] = user_agent
 
+    env['QUTE_USER_AGENT'] = websettings.user_agent()
     env['QUTE_CONFIG_DIR'] = standarddir.config()
     env['QUTE_DATA_DIR'] = standarddir.data()
     env['QUTE_DOWNLOAD_DIR'] = downloads.download_dir()
@@ -432,7 +446,7 @@ def run_async(tab, cmd, *args, win_id, env, verbose=False):
     cmd_path = os.path.expanduser(cmd)
 
     # if cmd is not given as an absolute path, look it up
-    # ~/.local/share/qutebrowser/userscripts (or $XDG_DATA_DIR)
+    # ~/.local/share/qutebrowser/userscripts (or $XDG_DATA_HOME)
     if not os.path.isabs(cmd_path):
         log.misc.debug("{} is no absolute path".format(cmd_path))
         cmd_path = _lookup_path(cmd)
@@ -443,6 +457,8 @@ def run_async(tab, cmd, *args, win_id, env, verbose=False):
     runner.finished.connect(commandrunner.deleteLater)
     runner.finished.connect(runner.deleteLater)
 
-    runner.prepare_run(cmd_path, *args, env=env, verbose=verbose)
+    runner.prepare_run(cmd_path, *args, env=env, verbose=verbose,
+                       output_messages=output_messages)
     tab.dump_async(runner.store_html)
     tab.dump_async(runner.store_text, plain=True)
+    return runner
